import {
  ProtocolRequest,
  ProtocolResponse,
  ProtocolEncoder,
  PacketBody,
  PacketBodyValue,
  ProtocolDecoder,
  ProtocolMessage
} from '@ucap-webmessenger/protocol';

export interface RoomOpenRequest extends ProtocolRequest {
  // 0. 구분자
  divCd: string;
  // 1n. 초대자 userSeq(n)...
  userSeqs: number[];
}

export interface RoomOpenResponse extends ProtocolResponse {
  // 0. 구분자
  divCd: string;
  // 1. 대화방SEQ(s)
  roomSeq: string;
  // 2.	신규여부(y)
  newRoom: boolean;
}

export const encodeRoomOpen: ProtocolEncoder<RoomOpenRequest> = (
  req: RoomOpenRequest
) => {
  const bodyList: PacketBody[] = [];

  bodyList.push({ type: PacketBodyValue.String, value: req.divCd });
  for (const userSeq of req.userSeqs) {
    bodyList.push({ type: PacketBodyValue.Integer, value: userSeq });
  }

  return bodyList;
};

export const decodeRoomOpen: ProtocolDecoder<RoomOpenResponse> = (
  message: ProtocolMessage
) => {
  return {
    divCd: message.bodyList[0],
    roomSeq: message.bodyList[1],
    newRoom: message.bodyList[2] === 'Y' ? true : false
  } as RoomOpenResponse;
};
